##############################################################################
#
# Copyright (c) 2004-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import fields, osv


class portal_portal(osv.osv):
	_name = "portal.portal"
	_description = "Portal"
	_columns = {
		'name': fields.char('Portal Name',size=64,required=True),
		'group_id': fields.many2one('res.groups', 'Associated Group',required=True),
		'menu_id': fields.many2one('ir.ui.menu','Main Menu', required=True),
		'menu_action_id': fields.many2one('ir.actions.act_window', 'User Menu Action', readonly=True,
										  help='''Default main menu for the users of the portal. This field is auto-completed at creation. '''),
		'home_action_id': fields.many2one('ir.actions.act_window', 'User Home Action', help='Complete this field to provide a Home menu different from the Main menu.'),
		'company_id': fields.many2one('res.company', 'Company', required=True),
		}

	def create_action_menu(self,cr,uid,action_id,action_name,context):
		"""
		Create default menu for the users of the portal.
		"""
		menu_action_id = self.pool.get('ir.actions.act_window').create(cr,uid, { 
			'name': action_name+ ' main menu',
			'usage': 'menu',
			'type':'ir.actions.act_window',
			'res_model': "ir.ui.menu",
			'domain': "[('parent_id','=',"+str(action_id)+")]",
			'view_type': 'tree',
			'view_id': self.pool.get('ir.model.data')._get_id(cr, uid, 'base', 'view_menu')
			})
		return menu_action_id

	
	def create(self, cr, user, vals, context={}):
		if not vals.get('menu_action_id') :
			vals['menu_action_id']= self.create_action_menu(cr,user,vals['menu_id'], vals['name'],context)
		return super(portal_portal, self).create(cr, user, vals, context)

	def create_menu(self, cr, uid,portal_id, portal_model_id, menu_name, action_id,parent_menu_id=None,context=None):
		"""
		Create a menuitem for the given portal and model whith the given name and action.
		"""
		
		assert portal_id and portal_model_id and menu_name and action_id, "Create_menu does not accept null values"
		
		portal= self.pool.get('portal.portal').browse(cr,uid,portal_id)
		model= self.pool.get('portal.model').browse(cr,uid,portal_model_id)
		action= self.pool.get('ir.actions.act_window').browse(cr,uid,action_id)
		## Create the menu:
		menu_id=self.pool.get('ir.ui.menu').create(cr, uid, {
			'name': menu_name,
			'parent_id': parent_menu_id or portal.menu_id.id,
			'icon': 'STOCK_NEW'
			})
		available_view={}
		for view in model.view_ids:
			available_view[view.type]= view.id
		vids = []
		i = 0
		## Fetch the views:
		for view in self.pool.get('ir.actions.act_window').browse(cr,uid,action_id).views:
			vids.append((0,0, {
				'sequence':i,
				'view_id': available_view.get(view[1], view[0]),
				'view_mode': view[1],
			}))
			i+=1

		## Duplicate the action and give the fetched views to the new one:
		action_id = action.copy(cr,uid, action.id,{
			'name': menu_name,
			'view_ids': vids
			})

		## Create the values:
		value_id = self.pool.get('ir.values').create(cr, uid, {
			'name': 'AutoGenerated by portal module',
			'key2': 'tree_but_open',
			'model': 'ir.ui.menu',
			'res_id': menu_id,
			'value': 'ir.actions.act_window,%d'%action_id,
			'object': True
			})
		## add the rule_group to the user
		if model.rule_group_id:
			portal.group_id.write(cr,uid,[portal.group_id.id],{'rule_groups': [(4,model.rule_group_id.id)]})
		return action_id
	
portal_portal()

	
class portal_model(osv.osv):
	_name = "portal.model"
	_description = "Portal Model"
	_columns = {
		'name': fields.char('Name',size=64,),
		'model_id': fields.many2one('ir.model','Model',required=True),
		'view_ids': fields.many2many('ir.ui.view','portal_model_view_rel','model_id','view_id','Views'),
		'rule_group_id': fields.many2one('ir.rule.group','Rule group'),
		}
portal_model()


